/* ----------------------------------------------------------------------------
  f2cad: Convert Fortran to C++ AD Types: Copyright (C) 2005-10 Bradley M. Bell
  Authors:  Bradley M. Bell    bradbell at washington dot edu
  License:  GNU Public License Version 2 or higher
-----------------------------------------------------------------------------*/
/*
$begin dgamma$$ 
$comment  The omhelp commands are documented in
	http://www.seanet.com/~bradbell/omhelp/
$$
$spell
	Mascheroni
	dgamma
	cpp
	hpp
	bool
	doublereal
	adolc
	cppad
	fadbad
$$

$section dgamma$$

$index dgamma$$

$head Prototype$$
$include p.omh/dgamma.p.omh$$

$children%
	f.omh/dgamma.f.omh
%$$
$head Fortran Source$$
$cref%dgamma.f%$$


$head Description$$
For arbitrary $latex x$$ and positive integer values of $latex \ell$$,
$latex \[
\begin{array}{rcl}
	\Gamma(x) & = & \int_0^\infty t^{x - 1} \exp( - t ) \; {\bf d} t
	\\
	\Gamma(\ell+1) & = & \ell !
	\\
	\Gamma^{(1)} (\ell+1) & = & 
		\ell ! \left( - \gamma + \sum_{k=1}^\ell \frac{1}{k} \right)
\end{array}
\] $$
where $latex \gamma$$ is the Euler-Mascheroni constant
$latex \[
	\gamma = 0.57721 56649 01532 86060 65120 90082 ...
\] $$


$codep */
# include <f2cad/dgamma.hpp>

test_result dgamma(void)
{	bool ok = true;

	// independent variables 
	integer     n  = 1;         // number of independent variables
	double     ell = 3.;
	doublereal  x[1];           // vector of independent variables
	x[0]          = ell+1.;     // value  of independent variables
	f2cad::Independent(n, x);   // declare x as independent variable vector

	// dependent variables 
	integer     m = 1;          // number of dependent variables
	doublereal  f[1];           // vector of dependent variables
	f[0] = f2cad::dgamma_(x+0); // compute value f[0] = Gamma(x[0])
	f2cad::Dependent(m, f);     // declare f as dependent variable vector

	// check the value of f
	ok &= f2cad::near_equal(f2cad::Value(f[0]), 6., 1e-10, 1e-10);

	// Compute derivative of f w.r.t x using f2cad common interface.
	// This general interface works with adolc, cppad, and fadbad.
	// See GetStarted for package specific more efficient code.
	double gamma = 0.577215664901532;
	double p     = f2cad::Partial<doublereal>(0, 0);
	double check = 6. * ( - gamma + 1. + 1./2. + 1./3. ); 
	ok &= f2cad::near_equal(p, check, 1e-10, 1e-10);

	if( ok )
		return test_pass;
	return test_fail;
}
/* $$
$end
*/
