/* ----------------------------------------------------------------------------
  f2cad: Convert Fortran to C++ AD Types: Copyright (C) 2005-10 Bradley M. Bell
  Authors:  Bradley M. Bell    bradbell at washington dot edu
  License:  GNU Public License Version 2 or higher
-----------------------------------------------------------------------------*/
/*
$begin near_equal$$
$spell
	cppad
	hpp
	std
	Cpp
	bool
	libfAD
	Adolc
	Fadbad
$$

$section Determine if Two Double Values Are Nearly Equal$$

$index double, near equal$$
$index equal, near$$
$index difference, absolute or relative$$
$index absolute, difference$$
$index relative, difference$$

$table
$bold Syntax$$ 
$cnext
$code # include <f2cad/near_equal.hpp>$$
$rnext $cnext
$syntax%bool near_equal(double %x%, double %y%, double %r%, double %a%)%$$
$tend


$head Description$$
Returns true if 
$latex \[
	x = y
	\; {\rm or} \;
	| x - y | \leq a
	\; {\rm or} \;
	\frac{ | x - y | } { |x| + |y| } \leq r
\] $$
Otherwise, it return false.
Note that $italic r$$ is a relative error bound and
$italic a$$ is an absolute error bound.

$head Linking$$
Each of the libraries 
$code libf2c_adolc.a$$,
$code libf2c_cppad.a$$,
and
$code libf2c_fadbad.a$$ has a copy of this routine in it.

$end
*/

namespace f2cad {
	bool near_equal(double x, double y, double r, double a)
	{
		if( x == y )
			return true;
	
		double ax = x;
		if( ax < 0. )
			ax = - ax;
	
		double ay = y;
		if( ay < 0. )
			ay = - ay;
	
		double ad = x - y;
		if( ad < 0. )
			ad = - ad;
	
		if( ad < a )
			return true;
	
		if( ad / (ax + ay) < r )
			return true;
	
		return false;
	}
} 
